import 'dart:convert';
import 'dart:io';
import 'dart:async';
import 'package:base_flutter/utils/text.dart';
import 'package:flutter/foundation.dart';

class Request {
  final String baseUrl;

  //StartPoint and EndPoint, Param
  String urlEnpoint;

  ///
  ///Param POST, GET
  ///[params] POST is Map<dynamic, dynamic>
  ///[params] GET is Map<String, String>
  ///
  dynamic params;

  Method method;

  Request({
    Key key,
    this.baseUrl,
    this.urlEnpoint,
    this.method = Method.GET,
  });

  Uri uri() {
    dynamic _temp;
    if (method == Method.POST ||
        method == Method.PUT ||
        method == Method.DELETE) {
      _temp = null; //params as Map<dynamic, dynamic>;
    } else {
      _temp = params as Map<String, String>;
    }
    // if (baseUrl.contains('test') || baseUrl.contains('pilot')) {
    //   return Uri.http(baseUrl, urlEnpoint, _temp);
    // } else {
    //   return Uri.https(baseUrl, urlEnpoint, _temp);
    // }
    return Uri.https(baseUrl, urlEnpoint, _temp);
  }
  
}
enum StatusRequest { SUCCESS, FAILURE, TIMEOUT }
enum Method { GET, POST, PUT, DELETE }

class ResultModel {
  String result;
  String message = '';
  String statusCode = '';
  String url = '';
  String messError = '';
  StatusRequest status;

  ResultModel(this.status, this.message);
}

class HttpUtility {
  // static final String baseUrl = Config.domain;
  Duration _timeOut = const Duration(seconds: 30);

  //Create Client
  final _httpClient = new HttpClient();

  _setHeaders(HttpHeaders headers) {
    headers.set('Content-Type', 'application/json');
  }

  Future<ResultModel> getRequest(Request _request) async {
    var uri = _request.uri();
    //init resultModel data default status "failure", message ""
    ResultModel resultModel = ResultModel(StatusRequest.FAILURE, '');

    try {
      var request = await _httpClient.getUrl(uri);
      _setHeaders(request.headers);

      print("URL REQUEST =============================>");
      print(_request.uri());
      print(_request.params);
      print(_request.method);
      print("<<======================================>>");

      var response = await request.close().timeout(_timeOut);

      if (response.statusCode >= 200 && response.statusCode < 300) {
        await response.transform(utf8.decoder).join().then((data) {
          resultModel.status = StatusRequest.SUCCESS;
          resultModel.result = data;
        });
      } else {
        resultModel.message = BeErrorText.connectServerError;
        // TrackingManager.server.requestFailure('get', _request.uri().toString());
      }

      return resultModel;
    } on TimeoutException catch (_) {
      //Timeout
      resultModel.status = StatusRequest.TIMEOUT;
      resultModel.message = BeErrorText.connectServerError;
      // TrackingManager.server.timeout('get', _request.uri().toString());
    } catch (error) {
      resultModel.message = BeErrorText.connectServerError;
      // TrackingManager.server.otherFailure('get', _request.uri().toString());
    }
    return resultModel;
  }

  Future<ResultModel> postRequest(Request _request) async {
    var uri = _request.uri();
    //init resultModel data default status "failure", message ""
    ResultModel resultModel = ResultModel(StatusRequest.FAILURE, '');

    try {
      var request = await _httpClient.postUrl(uri);
      _setHeaders(request.headers);
      // _setCookies(request);
      request.add(utf8.encode(json.encode(_request.params)));

      print("URL REQUEST =============================>");
      print(_request.uri());
      print(_request.params);
      print(_request.method);
      print(request.cookies);
      print("<<======================================>>");

      var response = await request.close().timeout(_timeOut);

      resultModel.statusCode = '${response?.statusCode ?? ""}';
      resultModel.messError = response?.headers?.contentType?.value ?? "";
      // resultModel.url = _request?.uri()?.toString() ?? "";

      if (response.statusCode >= 200 && response.statusCode < 300) {
        //success
        await response.transform(utf8.decoder).join().then((data) {
          resultModel.status = StatusRequest.SUCCESS;
          resultModel.result = data;
        });
      } else {
        //failure
        resultModel.message = BeErrorText.connectServerError;
        // TrackingManager.server
        //     .requestFailure('post', _request.uri().toString());
      }
    } on TimeoutException catch (e) {
      //Timeout
      resultModel.message = BeErrorText.connectServerError;
      // TrackingManager.server.timeout('post', _request.uri().toString());
    } on Exception catch (error) {
      //ERROR OTHER
      resultModel.url = error.toString();
      resultModel.message = BeErrorText.connectServerError;
      //error.toString()
      // TrackingManager.server.otherFailure('post', _request.uri().toString());
    }
    return resultModel;
  }

  Future<ResultModel> putRequest(Request _request) async {
    var uri = _request.uri();
    //init resultModel data default status "failure", message ""
    ResultModel resultModel = ResultModel(StatusRequest.FAILURE, '');

    try {
      var request = await _httpClient.putUrl(uri);
      _setHeaders(request.headers);
      request.add(utf8.encode(json.encode(_request.params)));

      print("URL REQUEST =============================>");
      print(_request.uri());
      print(_request.params);
      print(_request.method);
      print("<<======================================>>");

      var response = await request.close().timeout(_timeOut);

      if (response.statusCode >= 200 && response.statusCode < 300) {
        await response.transform(utf8.decoder).join().then((data) {
          resultModel.status = StatusRequest.SUCCESS;
          resultModel.result = data;
        });
      } else {
        resultModel.message = BeErrorText.connectServerError;
        // TrackingManager.server.requestFailure('put', _request.uri().toString());
      }

      return resultModel;
    } on TimeoutException catch (_) {
      //Timeout
      resultModel.status = StatusRequest.TIMEOUT;
      resultModel.message = BeErrorText.connectServerError;
      // TrackingManager.server.timeout('put', _request.uri().toString());
    } catch (error) {
      resultModel.message = BeErrorText.connectServerError;
      // TrackingManager.server.otherFailure('put', _request.uri().toString());
    }
    return resultModel;
  }

  Future<ResultModel> deleteRequest(Request _request) async {
    var uri = _request.uri();
    //init resultModel data default status "failure", message ""
    ResultModel resultModel = ResultModel(StatusRequest.FAILURE, '');

    try {
      var request = await _httpClient.deleteUrl(uri);
      _setHeaders(request.headers);

      print("URL REQUEST =============================>");
      print(_request.uri());
      print(_request.params);
      print(_request.method);
      print("<<======================================>>");

      var response = await request.close().timeout(_timeOut);

      if (response.statusCode >= 200 && response.statusCode < 300) {
        await response.transform(utf8.decoder).join().then((data) {
          resultModel.status = StatusRequest.SUCCESS;
          resultModel.result = data;
        });
      } else {
        resultModel.message = BeErrorText.connectServerError;
        // TrackingManager.server
        //     .requestFailure('delete', _request.uri().toString());
      }

      return resultModel;
    } on TimeoutException catch (_) {
      //Timeout
      resultModel.status = StatusRequest.TIMEOUT;
      resultModel.message = BeErrorText.connectServerError;
      // TrackingManager.server.timeout('delete', _request.uri().toString());
    } catch (error) {
      resultModel.message = BeErrorText.connectServerError;
      // TrackingManager.server.otherFailure('delete', _request.uri().toString());
    }
    return resultModel;
  }
}